# -*- coding: utf-8 -*-
from Acquisition import aq_parent
from Products.CMFPlone.utils import _createObjectByType
from os.path import dirname
from os.path import join
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping import tests
from plone.app.imagecropping.testing import PLONE_APP_IMAGECROPPING_INTEGRATION
from zope.annotation.interfaces import IAnnotations

import unittest


class TestCroppingAT(unittest.TestCase):

    layer = PLONE_APP_IMAGECROPPING_INTEGRATION

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']

        _createObjectByType('Image', self.portal, 'testimage',
                            title='I\'m a testing Image')

        self.img = self.portal.testimage
        f = file(join(dirname(tests.__file__), 'plone-logo.png'))
        self.img.setImage(f)
        f.close()

    def _jpegImage(self):
        """Convert our testimage to jpeg format and return it's data
        """

        from cStringIO import StringIO
        from PIL.Image import open

        img = open(file(join(dirname(tests.__file__), 'plone-logo.png')))
        out = StringIO()
        img.save(out, format='JPEG', quality=75)
        out.seek(0)
        result = out.getvalue()
        out.close()
        return result

    def test_image_and_annotation(self):
        """Check that our cropping view is able to store a cropped image
           and also saves the crop-box info in an annotation
        """

        view = self.img.restrictedTraverse('@@crop-image')
        traverse = self.portal.REQUEST.traverseName

        # check that the image scaled to thumb is not rectangular yet
        self.img.restrictedTraverse('')
        thumb = traverse(self.img, 'image_thumb')
        self.assertEqual((thumb.width, thumb.height), (128, 38))

        # there is also no annotations yet for cropped sizes on this image
        self.assertIsNone(IAnnotations(self.img).get(PAI_STORAGE_KEY),
                          'fresh image should not have any annotations')

        # store cropped version for thumb and check if the result
        # is a square now
        view._crop(fieldname='image', scale='thumb', box=(14, 14, 218, 218))
        thumb = traverse(self.img, 'image_thumb')
        self.assertEqual((thumb.width, thumb.height), (128, 128))

        # when storing a new crop-scale the crop-box information is
        # stored as an annotation.
        # this allows us to fire up the editor with the correct box when
        # editing the scale and it also allows us to identify if a scale
        # is cropped or simply resized
        self.assertEqual(IAnnotations(self.img).get(PAI_STORAGE_KEY).keys(),
                         ['image_thumb'],
                         "there's only one scale that is cropped")
        self.assertEqual(
            IAnnotations(self.img).get(PAI_STORAGE_KEY)['image_thumb'],
            (14, 14, 218, 218), 'wrong box information has been stored')

    def test_accessing_images(self):
        """Test if accessing the images works for our users
        """

        view = self.img.restrictedTraverse('@@crop-image')
        view._crop(fieldname='image', scale='thumb', box=(14, 14, 218, 218))
        traverse = self.portal.REQUEST.traverseName

        # one way to access the cropped image is via the traverser
        # <fieldname>_<scalename>
        thumb = traverse(self.img, 'image_thumb')
        self.assertEqual((thumb.width, thumb.height), (128, 128))

        # another is to use plone.app.imaging's ImageScaling view
        scales = traverse(self.img, '@@images')
        thumb2 = scales.scale('image', 'thumb')
        self.assertEqual(
            (thumb2.width, thumb2.height),
            (128, 128),
            'imagescaling does not return cropped image'
        )

    def test_image_formats(self):
        """make sure the scales have the same format as the original image
        """

        from cStringIO import StringIO
        from PIL.Image import open
        org_data = StringIO(self.img.getImage().data)
        self.assertEqual(open(org_data).format, 'PNG')

        view = self.img.restrictedTraverse('@@crop-image')
        view._crop(fieldname='image', scale='thumb', box=(14, 14, 218, 218))
        traverse = self.portal.REQUEST.traverseName
        cropped = traverse(self.img, 'image_thumb')
        croppedData = StringIO(cropped.data)
        self.assertEqual(
            open(croppedData).format,
            'PNG',
            'cropped scale does not have same format as the original'
        )

        # create a jpeg image out of the png file
        # and test if created scale is jpeg too
        _createObjectByType('Image', self.portal, 'testjpeg')
        jpg = self.portal.testjpeg
        jpg.setImage(self._jpegImage())

        org_data = StringIO(jpg.getImage().data)
        self.assertEqual(open(org_data).format, 'JPEG')

        view = jpg.restrictedTraverse('@@crop-image')
        view._crop(fieldname='image', scale='thumb', box=(14, 14, 218, 218))
        cropped = traverse(self.img, 'image_thumb')
        croppedData = StringIO(cropped.data)

        # XXX: fixme
        # self.assertEqual(open(croppedData).format, 'JPEG',
        #    "cropped scale does not have same format as the original")

    def test_modify_context(self):
        """ See https://github.com/collective/plone.app.imagecropping/issues/21
        """

        view = self.img.restrictedTraverse('@@crop-image')
        traverse = self.portal.REQUEST.traverseName
        scales = traverse(self.img, '@@images')
        unscaled_thumb = scales.scale('image', 'thumb')

        # store cropped version for thumb and check if the result
        # is a square now
        view._crop(fieldname='image', scale='thumb', box=(14, 14, 218, 218))

        # images accessed via context/@@images/image/thumb
        # stored in plone.scale annotation
        # see https://github.com/plone/plone.scale/pull/3#issuecomment-28597087
        thumb = scales.scale('image', 'thumb')
        self.assertNotEqual(thumb.data, unscaled_thumb.data)

        # images accessed via context/image_thumb
        # stored in attribute_storage
        thumb_attr = traverse(self.img, 'image_thumb')
        self.assertNotEqual(thumb_attr.data, unscaled_thumb.data)
        self.assertEqual(
            (thumb.width, thumb.height),
            (thumb_attr.width, thumb_attr.height)
        )

        self.img.setTitle('A new title')
        self.img.reindexObject()

        thumb2 = scales.scale('image', 'thumb')
        self.assertEqual(
            thumb.data,
            thumb2.data,
            'context/@@images/image/thumb accessor lost cropped scale'
        )

        thumb2_attr = traverse(self.img, 'image_thumb')
        self.assertEqual(
            (thumb.width, thumb.height),
            (thumb2_attr.width, thumb2_attr.height),
            'context/image_thumb accessor lost cropped scale'
        )

        # set a different image, this should invalidate scales
        self.img.setImage(self._jpegImage())

        jpeg_thumb_attr = traverse(self.img, 'image_thumb')
        self.assertNotEqual(
            (jpeg_thumb_attr.width, jpeg_thumb_attr.height),
            (128, 128),
            'context/image_thumb returns old cropped scale after '
            'setting a new image'
        )

        jpeg_thumb = scales.scale('image', 'thumb')
        self.assertNotEqual(
            (jpeg_thumb.width, jpeg_thumb.height),
            (128, 128),
            'context/@@images/image/thumb returns old cropped scale after '
            'setting a new image'
        )

    def test_copy_context(self):
        """ See https://github.com/collective/plone.app.imagecropping/issues/21
        """

        org_view = self.img.restrictedTraverse('@@crop-image')
        traverse = self.portal.REQUEST.traverseName
        org_scales = traverse(self.img, '@@images')
        org_unscaled_thumb = org_scales.scale('image', 'thumb')

        # store cropped version for thumb and check if the result
        # is a square now
        org_view._crop(
            fieldname='image',
            scale='thumb',
            box=(14, 14, 218, 218)
        )

        # images accessed via context/@@images/image/thumb
        # stored in plone.scale annotation
        # see https://github.com/plone/plone.scale/pull/3#issuecomment-28597087
        org_thumb = org_scales.scale('image', 'thumb')
        self.assertNotEqual(org_thumb.data, org_unscaled_thumb.data)

        # images accessed via context/image_thumb
        # stored in attribute_storage
        org_thumb_attr = traverse(self.img, 'image_thumb')
        self.assertNotEqual(org_thumb_attr.data, org_unscaled_thumb.data)
        self.assertEqual(
            (org_thumb.width, org_thumb.height),
            (org_thumb_attr.width, org_thumb_attr.height)
        )

        # Copy image
        container = aq_parent(self.img)
        container.manage_pasteObjects(
            container.manage_copyObjects(ids=[self.img.id, ]))

        # Be sure copied image is there
        self.assertIn('copy_of_testimage', container.objectIds())

        copy_img = container['copy_of_testimage']
        copy_scales = traverse(copy_img, '@@images')
        copy_thumb = copy_scales.scale('image', 'thumb')
        self.assertEqual(
            org_thumb.data,
            copy_thumb.data,
            'new thumb is different from old thumb scale after '
            'copying image (@@images access)'
        )

        copy_thumb_attr = traverse(copy_img, 'image_thumb')
        self.assertEqual(
            org_thumb_attr.data,
            copy_thumb_attr.data,
            'new thumb is different from old thumb scale after '
            'copying image (attribute access)'
        )

    def test_autocrop(self):
        """ check direction='down' for cropped and un-cropped image scales
        """
        traverse = self.portal.REQUEST.traverseName

        # auto-cropped scale
        img_scales = traverse(self.img, '@@images')
        auto_crop = img_scales.scale('image', 'thumb', direction='down')

        # cropped scale
        view = self.img.restrictedTraverse('@@crop-image')
        view._crop(fieldname='image', scale='thumb', box=(0, 0, 200, 200))
        manual_crop = img_scales.scale('image', 'thumb', direction='down')
        self.assertEqual(
            (auto_crop.width, auto_crop.height),
            (manual_crop.width, manual_crop.height))
        self.assertNotEqual(auto_crop.data, manual_crop.data)



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from plone.app.imagecropping import PRODUCT_NAME

UNINSTALL_PROFILE = 'profile-{0:s}:uninstall'.format(PRODUCT_NAME)


def uninstall(context, reinstall):
    if not reinstall:
        setup_tool = getToolByName(context, 'portal_setup')
        setup_tool.runAllImportStepsFromProfile(UNINSTALL_PROFILE)

    # remove controlpanel
    cp_tool = context['portal_controlpanel']
    cp_tool.unregisterApplication('ImageCroppingSettings')



# -*- coding: utf-8 -*-
from os.path import dirname
from os.path import join
from plone.app.imagecropping import tests
from plone.app.imagecropping.events import CroppingInfoChangedEvent
from plone.app.imagecropping.events import CroppingInfoRemovedEvent
from plone.app.imagecropping.interfaces import ICroppingInfoChangedEvent
from plone.app.imagecropping.interfaces import ICroppingInfoRemovedEvent
from plone.app.imagecropping.testing import PLONE_APP_IMAGECROPPING_FUNCTIONAL
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from zope.component import getGlobalSiteManager

import transaction
import unittest


class EditorTestCase(unittest.TestCase):

    layer = PLONE_APP_IMAGECROPPING_FUNCTIONAL

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']

        # setup testbrowser
        self.browser = Browser(self.app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic {0:s}:{1:s}'.format(TEST_USER_NAME, TEST_USER_PASSWORD)
        )
        self.createSingleImageType()

    def createSingleImageType(self):
        # create test image as testuser
        self.portal.invokeFactory('Image', 'testimage',
                                  title=u'I\'m a testing Image')
        self.img = self.portal['testimage']
        self.img.reindexObject()
        transaction.commit()

        f = file(join(dirname(tests.__file__), 'plone-logo.png'))
        self.img.setImage(f)
        f.close()

    def test_singleimage_editorview(self):
        # is there the cropping action tab
        self.browser.open('{0:s}/view'.format(self.img.absolute_url()))
        self.assertIn('Cropping', self.browser.contents)

        self.browser.getLink('Cropping').click()
        self.assertIn(u'Image Cropping Editor', self.browser.contents)

        # check for non existing image field column
        self.assertNotIn(u'Available Image Fields', self.browser.contents)

        # check for scales column
        self.assertTrue(u'Available Image Scales' in self.browser.contents)

        # check for editor buttons
        self.assertIn(u'Save cropping information', self.browser.contents)
        self.assertIn(u'Remove cropping information', self.browser.contents)

    def test_editview_crop(self, check_assert=True):
        scale_name = 'mini'
        request = self.layer['request']
        request.form.update({
            'x1': 1.0, 'y1': 2.7, 'x2': 10.6, 'y2': 8.4,
            'scalename': scale_name, 'form.button.Save': '1'})

        def get_cropped_scale(scales):
            return [s for s in scales if s['id'] == scale_name][0]

        cropview = self.img.restrictedTraverse('@@croppingeditor')
        cropview()
        cropped_scale = get_cropped_scale(cropview.scales())
        if check_assert:
            self.assertEqual(cropped_scale['is_cropped'], True)

        del request.form['form.button.Save']
        request.form.update({'form.button.Delete': '1'})
        cropview()
        removed_scale = get_cropped_scale(cropview.scales())
        if check_assert:
            self.assertEqual(removed_scale['is_cropped'], False)

    def test_events(self):
        sm = getGlobalSiteManager()
        firedEvents = []

        def recordEvent(event):
            firedEvents.append(event.__class__)

        sm.registerHandler(recordEvent, (ICroppingInfoChangedEvent,))
        sm.registerHandler(recordEvent, (ICroppingInfoRemovedEvent,))

        # do some cropping and removing
        self.test_editview_crop(check_assert=False)

        self.assertItemsEqual(firedEvents, [
            CroppingInfoChangedEvent,
            CroppingInfoRemovedEvent,
        ])
        sm.unregisterHandler(recordEvent, (ICroppingInfoChangedEvent,))
        sm.unregisterHandler(recordEvent, (ICroppingInfoRemovedEvent,))

    def tearDown(self):
        self.portal.manage_delObjects(['testimage', ])



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from OFS.Image import Pdata
from Products.ATContentTypes.interfaces.interfaces import IATContentType
from Products.Archetypes.interfaces.field import IImageField
from ZODB.blob import Blob
from plone.app.blob.config import blobScalesAttr
from plone.app.blob.interfaces import IBlobImageField
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.interfaces import IImageCroppingMarker
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.imaging.interfaces import IImageScaleHandler
from plone.app.imaging.traverse import ImageTraverser as BaseImageTraverser
from plone.scale.scale import scaleImage
from plone.scale.storage import AnnotationStorage
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.interface import implementer
from zope.publisher.interfaces import IRequest
import time


class IImageCroppingAT(IImageCroppingMarker):
    """Image cropping support marker interface for Archetypes based content
    """


def _millis():
    return int(time.time() * 1000)


@implementer(IImageCroppingUtils)
@adapter(IATContentType)
class CroppingUtilsArchetype(object):

    def __init__(self, context):
        self.context = context

    def image_fields(self):
        """ read interface
        """
        fields = []

        for field in self.context.Schema().fields():
            if IBlobImageField.providedBy(field) or IImageField.providedBy(field) and \
               field.get_size(self.context) > 0:
                fields.append(field)

        return fields

    def image_field_names(self):
        """ read interface
        """
        return [field.__name__ for field in self.image_fields()]

    def get_image_field(self, fieldname):
        """ read interface
        """
        return self.context.getField(fieldname)

    def get_image_label(self, fieldname):
        field = self.get_image_field(fieldname)
        return field.widget.label

    def get_image_data(self, fieldname):
        """ read interface
        """
        field = self.get_image_field(fieldname)
        blob = field.get(self.context)
        data = getattr(aq_base(blob), 'data', blob)
        if isinstance(data, Pdata):
            data = str(data)
        return data

    def get_image_size(self, fieldname):
        """ read interface
        """
        field = self.get_image_field(fieldname)
        image_size = field.getSize(self.context)
        return image_size

    def save_cropped(self, fieldname, scale, image_file):
        """ see interface
        """
        field = self.get_image_field(fieldname)
        handler = IImageScaleHandler(field)
        sizes = field.getAvailableSizes(self.context)
        w, h = sizes[scale]
        data = handler.createScale(
            self.context, scale, w, h, data=image_file.read())

        # store scale for classic <fieldname>_<scale> traversing
        handler.storeScale(self.context, scale, **data)

        # call plone.scale.storage.scale method in order to
        # provide saved scale for plone.app.imaging @@images view
        def crop_factory(fieldname, direction='keep', **parameters):
            blob = Blob()
            result = blob.open('w')
            _, image_format, dimensions = scaleImage(
                data['data'], result=result, **parameters)
            result.close()
            return blob, image_format, dimensions

        # Avoid browser cache
        # calling reindexObject updates the modified metadate too
        self.context.reindexObject()

        # call storage with actual time in milliseconds
        # this always invalidates old scales
        storage = AnnotationStorage(self.context, _millis)
        storage.scale(
            factory=crop_factory, fieldname=field.__name__, width=w, height=h)


@adapter(IImageCroppingAT, IRequest)
class ImageTraverser(BaseImageTraverser):
    """extend the standard image traverser to remove our cropping annotations
    (if present) in case the original image has been removed/replaced
    (no blobScalesAttr)
    """

    def publishTraverse(self, request, name):
        # remove scales information, if image has changed
        # We have no way of knowing if a non-blob image has changed, since we
        # don't have a time marker for when crops were generated.
        has_blobs = False
        for field in self.context.Schema().fields():
            if IBlobImageField.providedBy(field):
                has_blobs = True
                break

        if has_blobs and not hasattr(aq_base(self.context), blobScalesAttr) \
           and PAI_STORAGE_KEY in IAnnotations(self.context):
                del IAnnotations(self.context)[PAI_STORAGE_KEY]
        return super(ImageTraverser, self).publishTraverse(request, name)



# -*- coding: utf-8 -*-
from plone.app.imagecropping import PAI_STORAGE_KEY
from zope.annotation.interfaces import IAnnotations


class ScalingOverrides(object):

    _allow_rescale = True

    def _need_rescale(self, fieldname, scale):
        """If we've got a cropping annotation for the given fieldname
           and scale, set self._allow_rescale to False, to prevent
           plone.app.imaging traverser to overwrite our cropped scale

           since the self.modified() method does not know about the
           currently requested scale name, we need to use the _allow_rescale
           property
        """
        cropped = IAnnotations(self.context).get(PAI_STORAGE_KEY)
        if cropped and '{0:s}_{1:s}'.format(fieldname, scale) in cropped:
            self._allow_rescale = False
        else:
            self._allow_rescale = True



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from cStringIO import StringIO
from persistent.dict import PersistentDict
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.scale.storage import AnnotationStorage
from z3c.caching.purge import Purge
from zope.annotation.interfaces import IAnnotations
from zope.event import notify

import PIL.Image


class CroppingView(BrowserView):

    DEFAULT_FORMAT = 'PNG'

    def __call__(self, **kw):
        rq = self.request
        box = (rq['x1'], rq['y1'], rq['x2'], rq['y2'])

        self._crop(rq['fieldname'], rq['scale'], box)

        return True

    def _crop(self, fieldname, scale, box):
        """switch between dexterity and Archetypes
        """
        croputils = IImageCroppingUtils(self.context)
        data = croputils.get_image_data(fieldname)

        original_file = StringIO(data)
        image = PIL.Image.open(original_file)
        image_format = image.format or self.DEFAULT_FORMAT

        cropped_image = image.crop(box)
        cropped_image_file = StringIO()
        cropped_image.save(cropped_image_file, image_format, quality=100)
        cropped_image_file.seek(0)

        croputils.save_cropped(fieldname, scale, cropped_image_file)

        # store crop information in annotations
        self._store(fieldname, scale, box)

        # Purge caches if needed
        notify(Purge(self.context))

    @property
    def _storage(self):
        return IAnnotations(self.context).setdefault(
            PAI_STORAGE_KEY, PersistentDict())

    def _store(self, fieldname, scale, box):
        self._storage['{0:s}_{1:s}'.format(fieldname, scale)] = box
        return True

    def _read(self, fieldname, scale):
        return self._storage.get('{0:s}_{1:s}'.format(fieldname, scale))

    def _remove(self, fieldname, scale):
        # remove info from annotation
        key = '{0:s}_{1:s}'.format(fieldname, scale)
        if key in self._storage.keys():
            del self._storage[key]

        # remove saved scale
        scale_storage = AnnotationStorage(self.context)
        image_scales = self.context.restrictedTraverse('@@images')
        image_scale = image_scales.scale(fieldname, scale=scale)
        del scale_storage[image_scale.uid]



# -*- coding: utf-8 -*-
from zope.i18nmessageid.message import MessageFactory
import pkg_resources

try:
    pkg_resources.get_distribution('plone.namedfile')
except pkg_resources.DistributionNotFound:
    HAS_NAMEDFILE = False
else:
    HAS_NAMEDFILE = True

# TODO: backwards compatibility (probably not needed)
HAS_DEXTERITY = HAS_NAMEDFILE

imagecroppingMessageFactory = MessageFactory('plone.app.imagecropping')
PRODUCT_NAME = PAI_STORAGE_KEY = 'plone.app.imagecropping'



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from plone.app.imagecropping import PRODUCT_NAME
from plone.app.imagecropping.browser.settings import ISettings
from plone.registry.interfaces import IRegistry
from zope.component import queryUtility

import logging

logger = logging.getLogger('plone.app.imagecropping')
PROFILE_ID = 'profile-{0:s}:default'.format(PRODUCT_NAME)


def migrate0002to0003(context):
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')

    registry = queryUtility(IRegistry)
    settings = registry.forInterface(ISettings, check=False)
    if not hasattr(settings, 'constrain_cropping'):
        settings.constrain_cropping = False
    if not hasattr(settings, 'cropping_for'):
        settings.cropping_for = []
    if not hasattr(settings, 'default_cropping_for'):
        settings.default_cropping_for = []
    logger.info('Registry cleanup operation performed')
    logger.info('Migrated to profile version 0003')



from plone.app.imagecropping.interfaces import ICroppingInfoChangedEvent
from plone.app.imagecropping.interfaces import ICroppingInfoRemovedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class CroppingInfoChangedEvent(ObjectEvent):
    """ """
    implements(ICroppingInfoChangedEvent)


class CroppingInfoRemovedEvent(ObjectEvent):
    """ """
    implements(ICroppingInfoRemovedEvent)



# -*- coding: utf-8 -*-
from plone.app.imagecropping.testing import \
    PLONE_APP_IMAGECROPPING_ROBOT
from plone.testing import layered

import os
import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, 'robot')
    robot_tests = [os.path.join('robot', doc) for doc in
                   os.listdir(robot_dir) if doc.endswith('.robot') and
                   doc.startswith('test_')]
    for test in robot_tests:
        suite.addTests([
            layered(robotsuite.RobotTestSuite(test),
                    layer=PLONE_APP_IMAGECROPPING_ROBOT),
        ])
    return suite



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import getToolByName


PROFILE_ID = 'profile-plone.app.imagecropping:default'


def _cookResources(context):
    jstool = getToolByName(context, 'portal_javascripts')
    jstool.cookResources()
    csstool = getToolByName(context, 'portal_css')
    csstool.cookResources()


def to_0004(context):
    """search for iimagecropping interface in catalog and reindex those objects
    so they implment our new interfaces
    """
    iface = 'plone.app.imagecropping.interfaces.IImageCropping'
    cat = getToolByName(context, 'portal_catalog')

    # this consumes a lot of memory if we have too many objects to
    # update; we better use a generator to reduce memory usage and
    # avoid restarts on instances running with supervisor's memmon
    results = (b.getObject() for b in cat(object_provides=iface))
    for obj in results:
        obj.reindexObject(idxs=['object_provides'])



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from plone.app.imagecropping.testing import PLONE_APP_IMAGECROPPING_INTEGRATION

import unittest


class TestSetup(unittest.TestCase):

    layer = PLONE_APP_IMAGECROPPING_INTEGRATION

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.qi_tool = getToolByName(self.portal, 'portal_quickinstaller')

    def test_product_is_installed(self):
        """ Validate that our products GS profile has been run and the product
            installed
        """
        pid = 'plone.app.imagecropping'
        installed = [p['id'] for p in self.qi_tool.listInstalledProducts()]
        self.assertIn(pid, installed,
                      'package appears not to have been installed')

    def test_css_registered(self):
        cssreg = getattr(self.portal, 'portal_css')
        stylesheets_ids = cssreg.getResourceIds()
        self.assertIn(
            '++resource++plone.app.imagecropping.static/jquery.Jcrop.css',
            stylesheets_ids)
        self.assertIn(
            '++resource++plone.app.imagecropping.static/cropping.css',
            stylesheets_ids)



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from operator import itemgetter
from plone.app.imagecropping import imagecroppingMessageFactory as _
from plone.app.imagecropping.browser.settings import ISettings
from plone.app.imagecropping.events import CroppingInfoChangedEvent
from plone.app.imagecropping.events import CroppingInfoRemovedEvent
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.imaging.utils import getAllowedSizes
from plone.registry.interfaces import IRegistry
from zope import component
from zope.component._api import getUtility
from zope.event import notify
from zope.i18n import translate

import json


class CroppingEditor(BrowserView):
    """ Cropping Editor View """

    template = ViewPageTemplateFile('editor.pt')

    default_cropping_max_size = (0, 0)

    @property
    def croputils(self):
        return IImageCroppingUtils(self.context)

    @property
    def default_editor_size(self):
        return self._editor_settings.large_size.split(':')

    @property
    def showCropping(self):
        """returns True if there are any croppable scales on any of the fields
        """
        for field in self.image_field_names():
            if len(self.scales(field)) > 0:
                return True
        return False

    @property
    def fieldname(self):
        if 'fieldname' in self.request.form.keys():
            # TODO: check if requested fieldname is available
            # in self.image_fields
            return self.request.form.get('fieldname')

        img_field_names = self.image_field_names()
        return len(img_field_names) > 0 and img_field_names[0]

    def scales(self, fieldname=None):
        """Returns information to initialize JCrop for all available scales
           on the current content with the given fieldname and interface."""

        scales = []
        cropview = self.context.restrictedTraverse('@@crop-image')
        if fieldname is None:
            fieldname = self.fieldname
        image_size = self.croputils.get_image_size(fieldname)
        all_sizes = getAllowedSizes()
        current_selected = self.request.get('scalename', None)
        large_image_url = self.image_url(fieldname)
        constrain_cropping = self._editor_settings.constrain_cropping
        cropping_for = self._editor_settings.cropping_for
        for size, dim in sorted(all_sizes.iteritems(), key=itemgetter(1)):
            if constrain_cropping and size not in cropping_for:
                continue
            scale = dict()
            # scale jcrop config
            min_width, min_height = self._min_size(image_size, dim)
            max_width, max_height = self.default_cropping_max_size[0], \
                self.default_cropping_max_size[1]
            ratio_width, ratio_height = dim[0], dim[1]

            # lookup saved crop info
            select_box = cropview._read(fieldname, size)
            is_cropped = True

            if select_box is None:
                select_box = self._initial_size(image_size, dim)
                is_cropped = False

            config = dict([
                ('allowResize', True),
                ('allowMove', True),
                ('trueSize', [image_size[0], image_size[1]]),
                ('boxWidth', self.default_editor_size[0]),
                ('boxHeight', self.default_editor_size[1]),
                ('setSelect', select_box),
                ('aspectRatio', '%.2f' % (
                    float(ratio_width) / float(ratio_height))),
                ('minSize', [min_width, min_height]),
                ('maxSize', [max_width, max_height]),
                ('imageURL', large_image_url),
            ])
            scale['config'] = json.dumps(config)
            # scale value/id
            scale['id'] = size
            scale['title'] = '{0:s} {1:s}'.format(size, dim)
            scale['selected'] = size == current_selected and 'selected' or ''
            # flag if saved cropped scale was found
            # this helps to prevent generating unused
            # default scales in preview column
            scale['is_cropped'] = is_cropped
            # TODO: this is for thumbnail live-preview
            scale['thumb_width'] = ratio_width
            scale['thumb_height'] = ratio_height
            # safe original image url
            scale['image_url'] = large_image_url

            scales.append(scale)
        return scales

    def image_field_names(self):
        return self.croputils.image_field_names()

    def current_scale(self):
        """Returns information of the current selected scale"""
        images = self.scales()
        current_image = images[0]
        current = self.request.form.get('scalename', None)
        if isinstance(current, list):
            current = current[0]
        if current is not None:
            for image in images:
                if image['id'] == current:
                    current_image = image
        return current_image

    def current_url(self):
        """Returns the current page url"""
        context_state = component.getMultiAdapter(
            (self.context, self.request),
            name=u'plone_context_state'
        )
        return context_state.current_page_url()

    def image_url(self, fieldname):
        """Returns the url to the unscaled image"""
        scales = self.context.restrictedTraverse('@@images')
        scaled_img = scales.scale(
            fieldname,
            width=int(self.default_editor_size[0]),
            height=int(self.default_editor_size[1]),
            direction='keep',
        )
        return scaled_img and scaled_img.url or ''

    def field_label(self, fieldname):
        return self.croputils.get_image_label(fieldname)

    def _crop(self):
        coordinate = lambda x: int(round(float(self.request.form.get(x))))
        x1 = coordinate('x1')
        y1 = coordinate('y1')
        x2 = coordinate('x2')
        y2 = coordinate('y2')
        scale_name = self.request.form.get('scalename')
        cropping_util = self.context.restrictedTraverse('@@crop-image')
        cropping_util._crop(fieldname=self.fieldname,
                            scale=scale_name,
                            box=(x1, y1, x2, y2))

    def __call__(self):
        form = self.request.form
        if form.get('form.button.Delete', None) is not None:
            cropping_util = self.context.restrictedTraverse('@@crop-image')
            cropping_util._remove(self.fieldname, form.get('scalename'))
            notify(CroppingInfoRemovedEvent(self.context))
            IStatusMessage(self.request).add(_(u'Cropping area deleted'))
        if form.get('form.button.Save', None) is not None:
            self._crop()
            notify(CroppingInfoChangedEvent(self.context))
            IStatusMessage(self.request).add(
                _(u'Successfully saved cropped area'))

        # disable columns
        self.request.set('disable_plone.leftcolumn', 1)
        self.request.set('disable_plone.rightcolumn', 1)

        return self.template()

    def _min_size(self, image_size, scale_size):
        """ we need lower min-sizes if the image is smaller than the scale """
        width = scale_size[0]
        height = scale_size[1]
        if width > image_size[0]:
            ratio = float(image_size[0]) / float(width)
            width = image_size[0]
            height = float(height) * ratio
        if height > image_size[1]:
            ratio = float(image_size[1]) / float(height)
            height = image_size[1]
            width = float(width) * ratio
        return (int(round(width)), int(round(height)))

    def _initial_size(self, image_size, scale_size):
        """we need a best fit centered preselection to make editots life
        better.
        """
        ix, iy = map(float, image_size)

        # aspect ratio of original
        if iy > 0:
            ir = ix / iy
        else:
            ir = 1

        sx, sy = map(float, scale_size)

        # aspect ratio of scale
        if sy > 0:
            sr = sx / sy
        else:
            sr = 1

        # scale up to bounds
        if ir > sr:
            rx1, ry1 = ix * sr / ir, iy
        else:
            rx1, ry1 = ix, iy * ir / sr

        rx0, ry0 = 0, 0

        # center box
        if rx1 < ix:
            deltax = ix - rx1
            rx0 = deltax / 2
            rx1 = rx1 + deltax / 2
        if ry1 < iy:
            deltay = iy - ry1
            ry0 = deltay / 2
            ry1 = ry1 + deltay / 2

        # round to int
        rx0, ry0 = int(round(rx0)), int(round(ry0))
        rx1, ry1 = int(round(rx1)), int(round(ry1))
        return rx0, ry0, rx1, ry1

    @property
    def _editor_settings(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISettings)
        return settings

    @property
    def translated_confirm_discard_changes(self):
        # Escape for javascript
        return translate(
            _(u'Your changes will be lost. Continue?'),
            target_language=self.request.get('LANGUAGE', 'en'),
        ).replace('\'', '\\\'')



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import TEST_USER_ID
from plone.app.testing import applyProfile
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.testing import z2

import pkg_resources

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_DEXTERITY_CONTENTTYPES = False
else:
    HAS_DEXTERITY_CONTENTTYPES = True


class PloneAppImagecropping(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.imagecropping
        self.loadZCML('testing.zcml', package=plone.app.imagecropping)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.imagecropping:testing')
        portal.acl_users.userFolderAddUser('admin',
                                           'secret',
                                           ['Manager'],
                                           [])
        login(portal, 'admin')
        portal.portal_workflow.setDefaultChain('one_state_workflow')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        portal.invokeFactory(
            'Folder',
            id='acceptance-test-folder',
            title=u'Test Folder'
        )


class PloneAppImagecroppingDX(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        z2.installProduct(app, 'Products.DateRecurringIndex')
        if HAS_DEXTERITY_CONTENTTYPES:
            import plone.app.contenttypes
            self.loadZCML(package=plone.app.contenttypes)

        import plone.app.imagecropping
        self.loadZCML('testing.zcml', package=plone.app.imagecropping)

    def setUpPloneSite(self, portal):
        if HAS_DEXTERITY_CONTENTTYPES:
            applyProfile(portal, 'plone.app.contenttypes:default')

        applyProfile(portal, 'plone.app.imagecropping:testing')
        portal.acl_users.userFolderAddUser('admin',
                                           'secret',
                                           ['Manager'],
                                           [])
        login(portal, 'admin')
        portal.portal_workflow.setDefaultChain('one_state_workflow')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        portal.invokeFactory(
            'Folder',
            id='acceptance-test-folder',
            title=u'Test Folder'
        )


PLONE_APP_IMAGECROPPING = PloneAppImagecropping()
PLONE_APP_IMAGECROPPING_INTEGRATION = IntegrationTesting(
    bases=(PLONE_APP_IMAGECROPPING, ),
    name='plone.app.imagecropping:Integration')
PLONE_APP_IMAGECROPPING_FUNCTIONAL = FunctionalTesting(
    bases=(PLONE_APP_IMAGECROPPING, ),
    name='plone.app.imagecropping:Functional')
PLONE_APP_IMAGECROPPING_DX = PloneAppImagecroppingDX()
PLONE_APP_IMAGECROPPING_INTEGRATION_DX = IntegrationTesting(
    bases=(PLONE_APP_IMAGECROPPING_DX, ),
    name='plone.app.imagecropping:Integration_DX')
PLONE_APP_IMAGECROPPING_FUNCTIONAL_DX = FunctionalTesting(
    bases=(PLONE_APP_IMAGECROPPING_DX, ),
    name='plone.app.imagecropping:Functional_DX')
PLONE_APP_IMAGECROPPING_ROBOT = FunctionalTesting(
    bases=(
        AUTOLOGIN_LIBRARY_FIXTURE,
        PLONE_APP_IMAGECROPPING,
        z2.ZSERVER_FIXTURE
    ),
    name='plone.app.imagecropping:Robot')



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

version = '1.3'

setup(
    name='plone.app.imagecropping',
    version=version,
    description="allows images to be manually cropped using JCrop JS library",
    long_description='\n\n'.join([
        open('README.rst').read(),
        open('CONTRIBUTORS.rst').read(),
        open('CHANGES.rst').read(),
    ]),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.2",
        "Framework :: Plone :: 4.3",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='plone image crop',
    author='',
    author_email='',
    url='https://github.com/collective/plone.app.imagecropping',
    license='GPLv2',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Acquisition',
        'Pillow',
        'plone.app.blob',
        'plone.app.imaging',
        'plone.app.registry',
        'plone.behavior',
        'plone.namedfile>=2.0.1',
        'plone.registry',
        'plone.scale',
        'Products.CMFCore',
        'Products.CMFPlone>=4.2'
        'Products.GenericSetup',
        'setuptools',
        'zope.component',
        'zope.globalrequest',
        'zope.interface',
        'zope.lifecycleevent',
        'zope.schema',
        'z3c.caching',
    ],
    extras_require={
        'test': [
            'AccessControl',
            'plone.app.dexterity',
            'plone.app.robotframework',
            'plone.app.testing[robot]>=4.2.2',
            'plone.testing',
            'robotsuite',
            'zope.event',
        ],
        'archetypes': [
            'Products.ATContentTypes',
        ],
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
